--
-- consoleswitch
--
-- Select which of two UARTs will be connected to a
-- physical output pin.

-- DO NOT EDIT BELOW THIS LINE --------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library proc_common_v2_00_a;
use proc_common_v2_00_a.proc_common_pkg.all;

-- DO NOT EDIT ABOVE THIS LINE --------------------

--USER libraries added here

------------------------------------------------------------------------------
-- Entity section
------------------------------------------------------------------------------
-- Definition of Generics:
--   C_SLV_DWIDTH                 -- Slave interface data bus width
--   C_NUM_REG                    -- Number of software accessible registers
--
-- Definition of Ports:
--   Bus2IP_Clk                   -- Bus to IP clock
--   Bus2IP_Reset                 -- Bus to IP reset
--   Bus2IP_Addr                  -- Bus to IP address bus
--   Bus2IP_Data                  -- Bus to IP data bus
--   Bus2IP_BE                    -- Bus to IP byte enables
--   Bus2IP_RdCE                  -- Bus to IP read chip enable
--   Bus2IP_WrCE                  -- Bus to IP write chip enable
--   IP2Bus_Data                  -- IP to Bus data bus
--   IP2Bus_RdAck                 -- IP to Bus read transfer acknowledgement
--   IP2Bus_WrAck                 -- IP to Bus write transfer acknowledgement
--   IP2Bus_Error                 -- IP to Bus error response
------------------------------------------------------------------------------

entity user_logic is
  generic
  (
    -- ADD USER GENERICS BELOW THIS LINE ---------------
    --USER generics added here
    -- ADD USER GENERICS ABOVE THIS LINE ---------------

    -- DO NOT EDIT BELOW THIS LINE ---------------------
    -- Bus protocol parameters, do not add to or delete
    C_SLV_DWIDTH                   : integer              := 32;
    C_NUM_REG                      : integer              := 4
    -- DO NOT EDIT ABOVE THIS LINE ---------------------
  );
  port
  (
    -- ADD USER PORTS BELOW THIS LINE ------------------
    --USER ports added here
      uart0tx               : in std_logic ;
      uart0rx               : out std_logic ;
      uart1tx               : in std_logic ;
      uart1rx               : out std_logic ;
      ext0tx                : out std_logic ;
      ext0rx                : in std_logic ;
      ext1tx                : out std_logic ;
      ext1rx                : in std_logic ;
    -- ADD USER PORTS ABOVE THIS LINE ------------------

    -- DO NOT EDIT BELOW THIS LINE ---------------------
    -- Bus protocol ports, do not add to or delete
    Bus2IP_Clk                     : in  std_logic;
    Bus2IP_Reset                   : in  std_logic;
    Bus2IP_Addr                    : in  std_logic_vector(0 to 31);
    Bus2IP_Data                    : in  std_logic_vector(0 to C_SLV_DWIDTH-1);
    Bus2IP_BE                      : in  std_logic_vector(0 to C_SLV_DWIDTH/8-1);
    Bus2IP_RdCE                    : in  std_logic_vector(0 to C_NUM_REG-1);
    Bus2IP_WrCE                    : in  std_logic_vector(0 to C_NUM_REG-1);
    IP2Bus_Data                    : out std_logic_vector(0 to C_SLV_DWIDTH-1);
    IP2Bus_RdAck                   : out std_logic;
    IP2Bus_WrAck                   : out std_logic;
    IP2Bus_Error                   : out std_logic
    -- DO NOT EDIT ABOVE THIS LINE ---------------------
  );

  attribute SIGIS : string;
  attribute SIGIS of Bus2IP_Clk    : signal is "CLK";
  attribute SIGIS of Bus2IP_Reset  : signal is "RST";

end entity user_logic;

------------------------------------------------------------------------------
-- Architecture section
------------------------------------------------------------------------------

architecture IMP of user_logic is

    signal select_zero : std_logic ;

begin
    process ( Bus2IP_Reset, Bus2IP_Clk ) is
    begin
        if ( Bus2IP_Reset = '1' )
        then
            select_zero <= '1' ;
            ext0tx <= '1' ;
            ext1tx <= '1' ;
            uart0rx <= '1' ;
            uart1rx <= '1' ;
        elsif (( Bus2IP_Clk'event )
        and ( Bus2IP_Clk = '1' ))
        then
            if ( Bus2IP_WrCE ( 0 ) = '1' )
            then
                select_zero <= Bus2IP_Data ( C_SLV_DWIDTH - 1 ) ;
            end if;
            if ( select_zero = '1' )
            then
                -- Normal state: passthrough with register
                ext0tx <= uart0tx ;
                uart0rx <= ext0rx ;
                ext1tx <= uart1tx ;
                uart1rx <= ext1rx ;
            else 
                -- Alternate state: swap 
                ext1tx <= uart0tx ;
                uart0rx <= ext1rx ;
                ext0tx <= uart1tx ;
                uart1rx <= ext0rx ;
            end if;
        end if;
    end process;

    IP2Bus_Data  <= ( others => '0' ) ;
    IP2Bus_WrAck <= (Bus2IP_WrCE(0) or 
                    Bus2IP_WrCE(1) or Bus2IP_WrCE(2) or Bus2IP_WrCE(3));
    IP2Bus_RdAck <= (Bus2IP_RdCE(0) or 
                    Bus2IP_RdCE(1) or Bus2IP_RdCE(2) or Bus2IP_RdCE(3));
    IP2Bus_Error <= '0';

end IMP;
